/**
 * \file
 * \version		1.0
 * \date 		2013-10-06
 * \author		Joel Guittet - http://myfreescalewebpage.free.fr
 * 
 * \brief		This file is used to access to the EEPROM
 * 
 * \verbatim
 * 
 * EEPROM Application Note
 * 
 * Copyright (C) 2013 MyFreescaleWebPage
 *
 * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Change History
 * 
 * Version	| Author		| Description
 * ----------------------------------------------------------------------------------------------------------------------------------------
 * 1.0		| Joel Guittet	| Initial Release
 * 1.1      | Joel Guittet	| Port to CodeWarrior 10.5, rewrite application note
 * 
 * \endverbatim
 */


/*---------------------------------------------------------------------------------------------------------------------------------------*/
/* Includes                                                                                                                              */
/*---------------------------------------------------------------------------------------------------------------------------------------*/

#include "eeprom.h"


/*---------------------------------------------------------------------------------------------------------------------------------------*/
/* Functions                                                                                                                             */
/*---------------------------------------------------------------------------------------------------------------------------------------*/

/**
 * This function initializes the EEPROM - Must be called on startup
 */
void EEPROM_Init(void)
{
	/* Select page 0 */
	FCNFG_EPGSEL = 0;
	
	/* Set the EEPROM clock to 200kHz (Fbus = 18MHz) */
	FCDIV = FCDIV_PRDIV8_MASK | 0x0B;
}

/**
 * This function reads a single sector of the EEPROM (8 bytes)
 * @param u16Address Address of the sector, 0 is the beginning of the memory, must be a multiple of 8 bytes 
 * @param pBuffer 8 bytes buffer where to store the data from the EEPROM 
 * @return TRUE if the memory was read, FALSE otherwise
 */
BOOLEAN EEPROM_Read(UINT16 u16Address, UINT8* pBuffer)
{
	UINT8 u8Index;
  
	/* Check pBuffer */
	if (pBuffer == NULL) return FALSE;
	
	/* Check u16Address */
	if ((u16Address >= EEPROM_SIZE) || ((u16Address % 8) != 0)) return FALSE;
		
	/* Select the right page of the EEPROM */
	if (u16Address >= PAGE_SIZE)
	{
		/* PAGE_SIZE <= u16Address < EEPROM_SIZE => select page 1 */
		FCNFG_EPGSEL = 1;
		u16Address -= PAGE_SIZE;
	}
	else
	{
		/* 0 <= u16Address < PAGE_SIZE => select page 0 */
		FCNFG_EPGSEL = 0;
	}
    
	/* Translate address value according to the mapping of the microcontroller */
	u16Address += EEPROM_ADDRESS;
  
	for (u8Index = 0; u8Index < 8; u8Index++)
	{
		*(pBuffer + u8Index) = *((UINT8*)u16Address + u8Index);
	}
	
	return TRUE;
}

/**
 * This function writes a single sector of the EEPROM (8 bytes)
 * @param u16Address Address of the sector, 0 is the beginning of the memory, must be a multiple of 8 bytes 
 * @param pBuffer 8 bytes buffer where to get the data to be stored the EEPROM 
 * @return TRUE if the memory was written, FALSE otherwise
 */
BOOLEAN EEPROM_Write(UINT16 u16Address, UINT8* pBuffer)
{
	UINT8 u8Index;
	BOOLEAN bResult = TRUE;
	
	/* Check pBuffer */
	if (pBuffer == NULL) return FALSE;
	
	/* Check u16Address */
	if ((u16Address >= EEPROM_SIZE) || ((u16Address % 8) != 0)) return FALSE;
		
	/* Select the right page of the EEPROM */
	if (u16Address >= PAGE_SIZE)
	{
		/* PAGE_SIZE <= u16Address < EEPROM_SIZE => select page 1 */
		FCNFG_EPGSEL = 1;
		u16Address -= PAGE_SIZE;
	}
	else
	{
		/* 0 <= u16Address < PAGE_SIZE => select page 0 */
		FCNFG_EPGSEL = 0;
	}
	
	/* Translate address value according to the mapping of the microcontroller */
	u16Address += EEPROM_ADDRESS;
  
	/* It is now possible to write into the EEPROM - First erase the sector */   
	FSTAT = 0x30;                   					/* Clear possible errors conditions */  
	*(UINT8*)u16Address = 0xFF;							/* Write a single data byte, for erase operation, value is not important */
	FCMD = 0x40;                    					/* Sector Erase command */
	FSTAT_FCBEF = 1;                					/* Launch the command */
	
	/* Check if an error occurred */
	if ((FSTAT_FPVIOL == 1) || (FSTAT_FACCERR == 1)) bResult = FALSE;
  
	/* Wait operation completed */
	while (!FSTAT_FCCF);
  
	/* Erase operation completed, now write the sector of the EEPROM */
	
	/* Clear possible errors conditions */
	FSTAT = 0x30;
	
	for (u8Index = 0; u8Index < 8; u8Index++)
	{
		/* Write data */
		*((UINT8*)u16Address + u8Index) = *(pBuffer + u8Index);
		
		FCMD = 0x25;                    				/* Burst program command */
		FSTAT_FCBEF = 1;                				/* Launch the command */
		
		/* Check if an error occurs */
		if ((FSTAT_FPVIOL == 1) || (FSTAT_FACCERR == 1)) bResult = FALSE;
		
		/* Wait operation completed */
		while (!FSTAT_FCCF);
		while (!FSTAT_FCBEF);
	}
	
	return bResult;
}

/**
 * This function erases all the EEPROM
 * @return TRUE if the memory was erased, FALSE otherwise
 */
BOOLEAN EEPROM_Erase(void)
{
	BOOLEAN bResult = TRUE;
	UINT16 u16Address = EEPROM_ADDRESS;
	
	FSTAT = 0x30;                   					/* Clear possible errors conditions */  
	*(UINT8*)u16Address = 0xFF;							/* Write a single data byte, for erase operation, value is not important */
	FCMD = 0x41;                    					/* Mass Erase command */
	FSTAT_FCBEF = 1;                					/* Launch the command */
		
	/* Check if an error occurs */
	if ((FSTAT_FPVIOL == 1) || (FSTAT_FACCERR == 1)) bResult = FALSE;
	
	/* Wait operation completed */
	while (!FSTAT_FCCF);
	
	return bResult;
}
