/**
 * \file
 * \version		1.0
 * \date 		2013-10-06
 * \author		Joel Guittet - http://myfreescalewebpage.free.fr
 * 
 * \brief		This file contains the main entry point of the application
 * 
 * \verbatim
 * 
 * EEPROM Application Note
 * 
 * Copyright (C) 2013 MyFreescaleWebPage
 *
 * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Change History
 * 
 * Version	| Author		| Description
 * ----------------------------------------------------------------------------------------------------------------------------------------
 * 1.0		| Joel Guittet	| Initial Release
 * 1.1      | Joel Guittet	| Port to CodeWarrior 10.5, rewrite application note 
 * 
 * \endverbatim
 */


/*---------------------------------------------------------------------------------------------------------------------------------------*/
/* Includes                                                                                                                              */
/*---------------------------------------------------------------------------------------------------------------------------------------*/

#include <hidef.h>
#include "derivative.h"
#include "typedef.h"
#include "eeprom.h"


/*---------------------------------------------------------------------------------------------------------------------------------------*/
/* Includes                                                                                                                              */
/*---------------------------------------------------------------------------------------------------------------------------------------*/

static void MCG_Init(void);


/*---------------------------------------------------------------------------------------------------------------------------------------*/
/* Global Variables                                                                                                                      */
/*---------------------------------------------------------------------------------------------------------------------------------------*/

/* Buffer used to read the EEPROM */
UINT8 MAIN_EepromBufferRead[8] = 
{
	0, 0, 0, 0, 0, 0, 0, 0
};

/* Buffer used to write in the EEPROM */
UINT8 MAIN_EepromBufferWrite[8] =
{
	'-', 'E', 'E', 'P', 'R', 'O', 'M', '-'
};

/* Store the status of the operations on the EEPROM */
BOOLEAN MAIN_EepromStatus = FALSE;


/*---------------------------------------------------------------------------------------------------------------------------------------*/
/* Functions                                                                                                                             */
/*---------------------------------------------------------------------------------------------------------------------------------------*/

/**
 * Main function, called on startup. Never return.
 */
void main(void)
{
	MCG_Init();							/* Initialize clock */
	EEPROM_Init();						/* Initialize EEPROM */
		
	EnableInterrupts;
	 
	/* 1- Erase all the EEPROM */
	MAIN_EepromStatus = EEPROM_Erase();

	/* Breakpoint */
	asm(bgnd);
		
	/* Have a look to MAIN_EepromStatus */
	/* -> MAIN_EepromStatus = TRUE in case of success */
	/* -> Here you can see that the internal EEPROM is erased, using the memory view, at address 0x1400 (all bytes set to 0xFF) */

	/* 2- Reading the first sector of the EEPROM */
	MAIN_EepromStatus = EEPROM_Read(0, MAIN_EepromBufferRead);

	/* Breakpoint */
	asm(bgnd);
		
	/* Have a look to MAIN_EepromBufferRead and MAIN_EepromStatus */
	/* -> MAIN_EepromStatus = TRUE in case of success */
	/* -> Here you can see that the content of the array MAIN_EepromBufferRead is 0xFF */

	/* 3- Writing in the first sector of the EEPROM */
	MAIN_EepromStatus = EEPROM_Write(0, MAIN_EepromBufferWrite);

	/* Breakpoint */
	asm(bgnd);
		
	/* Have a look to MAIN_EepromStatus variable */
	/* -> MAIN_EepromStatus = TRUE in case of success */
	/* -> Here you can see the new content of the EEPROM, using the memory view, at the address 0x1400 */

	/* 4- Reading again the first sector of the EEPROM */
	MAIN_EepromStatus = EEPROM_Read(0, MAIN_EepromBufferRead);

	/* Breakpoint */
	asm(bgnd);
		
	/* Have a look to MAIN_EepromBufferRead and MAIN_EepromStatus */
	/* -> MAIN_EepromStatus = TRUE in case of success */
	/* -> Here you can see that the content of the array MAIN_EepromBufferRead is now "-EEPROM-" (you can display the content in ASCII mode in the debugger) */

	/* 5- Just to see an error using the EEPROM functions, try to read at an address which is not a multiple of 8 */
	MAIN_EepromStatus = EEPROM_Read(1, MAIN_EepromBufferRead);

	/* Breakpoint */
	asm(bgnd);
		
	/* Have a look to g_ucBufferRead and g_bStatus */
	/* -> MAIN_EepromStatus = FALSE because an error occurred */
	/* -> The content of MAIN_EepromBufferRead was not changed */
	
	/* Main loop */
	for (;;)
	{
		/* Reset the watchdog */
		__RESET_WATCHDOG();
	}
}

/**
 * This functions initializes the MCG peripheral (FEI, crystal: 12MHz, clock: 18MHz)
 */
static void MCG_Init(void)
{
	MCGC2 = 0x36;
	while (MCGSC_OSCINIT != 1){}
	MCGC1 = 0xB8;
	while (MCGSC_IREFST != 0){}
	while (MCGSC_CLKST != 0b10){}  
	MCGC1 = 0x98;
	MCGC3 = 0x46;
	while (MCGSC_PLLST != 1){}
	while (MCGSC_LOCK != 1){}
	MCGC1 = 0x18;
	while (MCGSC_CLKST != 0b11){}
}
